/*
* au-string.c - Minimal linked list library for strings
* Copyright (c) 2014 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved. 
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#include "au-string.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>	// for dump()


void slist_create(slist *l)
{
	l->head = NULL;
	l->cur = NULL;
	l->cnt = 0;
}

static void slist_last(slist *l)
{
        register snode* cur;
	
	if (l->head == NULL)
		return;

	// Try using cur so that we don't have to start at beginnning
	if (l->cur)
		cur = l->cur;
	else
	        cur = l->head;

	// Loop until no next value
	while (cur->next)
		cur = cur->next;
	l->cur = cur;
}

snode *slist_next(slist *l)
{
	if (l->cur == NULL)
		return NULL;
	l->cur = l->cur->next;
	return l->cur;
}

static void slist_append(slist *l, snode *node)
{
	snode* newnode;

	newnode = malloc(sizeof(snode));

	if (node->str)
		newnode->str = strdup(node->str);
	else
		newnode->str = NULL;

	newnode->type = node->type;
	newnode->next = NULL;

	if (newnode->type != FIELD_LITERAL) {
		// Reclassify these
		if (strcmp(newnode->str, "date") == 0)
			newnode->type = FIELD_VIRTUAL;
		else if (strcmp(newnode->str, "time") == 0)
			newnode->type = FIELD_VIRTUAL;
		else if (strcmp(newnode->str, "milli") == 0)
			newnode->type = FIELD_VIRTUAL;
		else if (strcmp(newnode->str, "serial") == 0)
			newnode->type = FIELD_VIRTUAL;
	}

	// Make sure cursor is at the end
	slist_last(l);

	// if we are at top, fix this up
	if (l->head == NULL)
		l->head = newnode;
	else	// Otherwise add pointer to newnode
		l->cur->next = newnode;

	// make newnode current
	l->cur = newnode;
	l->cnt++;
}

void slist_clear(slist* l)
{
	snode* nextnode;
	register snode* current;

	current = l->head;
	while (current) {
		nextnode=current->next;
		free((void *)current->str);
		free(current);
		current=nextnode;
	}
	l->head = NULL;
	l->cur = NULL;
	l->cnt = 0;
}

int slist_load(slist *l, char *f)
{
	snode s;
	char *ptr = f;
	const char *str = ptr;
	int mark, done;	// mark means we are doing a field name

	done = 0;
	if (*ptr == '%') {
		mark = 1;
		ptr++;
		str = ptr;
	} else
		mark = 0;
	do {
		if (mark == 0) {
			// Check until field name
			while (*ptr != '%' && *ptr != 0)
				ptr++;

			if (*ptr == '%')
				mark = 1;
			else if (*ptr == 0)
				done = 1;

			if (*ptr)
				*ptr = 0; // This destroys the %
			s.type = FIELD_LITERAL;
			s.str = str;
			slist_append(l, &s);
			if (!done) {
				ptr++;
				str = ptr; // We now point after the %
			}
		} else {
			char term;
			if (*ptr == '%') {
				// We hit double % treat as literal
				ptr++;
				term = *ptr;
				*ptr = 0;
				s.type = FIELD_LITERAL;
				s.str = str;
				slist_append(l, &s);
				*ptr = term;
				str = ptr;
				// Check what's next
				if (*str == 0)
					done = 1;
				else if (*str != '%')
					mark = 0;
				else {
					ptr++;
					str = ptr;
				}
				continue;
			}
			// We are parsing the %field
			while (isalpha(*ptr)) {
				ptr++;
			}

			// We should be pointing one after field
			if (*ptr == 0)
				done = 1;
			else {
				term = *ptr;
				*ptr = 0;
			}
			// Decide which type
			if (isupper(*str)) {
				s.type = FIELD_INTERPRET;
				char *tmp = (char *)str;
				while (*tmp) {
					*tmp = tolower(*tmp);
					tmp++;
				}
			} else
				s.type = FIELD_LOOKUP;
			s.str = str;
			slist_append(l, &s);

			if (!done) {
				*ptr = term;
				str = ptr;
				// Check what's next
				if (term != '%')
					mark = 0;
				else {
					ptr++;
					str = ptr;
				}
			}
		}
	} while (!done);

	return 1;
}

void slist_dump(slist *l)
{
	snode *f;
	slist_first(l);
	while ((f = slist_get_cur(l))) {
		switch (f->type) {
			case FIELD_LITERAL:
				printf("literal:%s.\n", f->str);
				break;
			case FIELD_LOOKUP:
				printf("lookup:%s.\n", f->str);
				break;
			case FIELD_INTERPRET:
				printf("interpret:%s.\n", f->str);
				break;
			case FIELD_VIRTUAL:
				printf("virtual:%s.\n", f->str);
				break;
			default:
				break;
		}
		slist_next(l);
	}
}
