/*
 * auformat.c - Utility to extract and interpret fields
 * Copyright (c) 2014 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This software may be freely redistributed and/or modified under the
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Authors:
 *   Steve Grubb <sgrubb@redhat.com>
 */

#include <stdio.h>
#include <locale.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <auparse.h>
#include "au-string.h"

static slist format;
static const char *path = NULL;
static int fill = 0;

static void print_virtual(auparse_state_t *au, snode *s)
{
	if (strcmp(s->str, "date") == 0) {
		char str[48];
		time_t t = auparse_get_time(au);
		struct tm *tv = localtime(&t);
		strftime(str, sizeof(str), "%x", tv);
		fputs(str, stdout);
	} else if (strcmp(s->str, "time") == 0) {
		char str[32];
		time_t t = auparse_get_time(au);
		struct tm *tv = localtime(&t);
		strftime(str, sizeof(str), "%T", tv);
		fputs(str, stdout);
	} else if (strcmp(s->str, "milli") == 0) {
		printf("%u", auparse_get_milli(au));
	} else if (strcmp(s->str, "serial") == 0) {
		printf("%lu", auparse_get_serial(au));
	}
}

static void print_item(auparse_state_t *au, snode *s)
{
	const char *str, *ptr;
	auparse_first_record(au);

	switch (s->type) {
		case FIELD_LITERAL:
			// FIXME: This can be sped up by adding a field in
			// the snode to denote if a \n is in the string. If
			// so take this long way, otherwise call fputs()
			ptr = s->str;
			do {
				if (*ptr == '\\') {
					if (*(ptr+1) == 'n') {
						printf("\n");
						ptr++;
					} else if (*(ptr+1) == 't') {
						printf("\t");
						ptr++;
					} else
						putchar(*ptr);
				} else
					putchar(*ptr);
				ptr++;
			} while (*ptr);
			break;
		case FIELD_LOOKUP:
			str = auparse_find_field(au, s->str);
			if (str)
				fputs(str, stdout);
			break;
		case FIELD_INTERPRET:
			str = auparse_find_field(au, s->str);
			if (str) {
				str = auparse_interpret_field(au);
				if (str)
					fputs(str, stdout);
			} else {
				if (fill)
					fputs("(none)", stdout);
				else
					fprintf(stderr,	"Field %s not found\n",
								s->str);
			}
			break;
		case FIELD_VIRTUAL:
			print_virtual(au, s);
			break;
		default:
			break;
	}
}


int main(int argc, char *argv[])
{
	int n = 1;
	auparse_state_t *au;

	setlocale (LC_ALL, "");
	if (argc < 2) {
	    fprintf(stderr,
		"Usage: auformat <format-string> [path-to-log] [--fill]\n");
	    return 1;
	}

	slist_create(&format);
	while (argv[n]) {
		if (strcmp(argv[n], "--fill") == 0)
			fill = 1;
		else if (access(argv[n], R_OK) == 0)
			path = argv[n];
		else
			slist_load(&format, argv[n]);
		n++;
	}	
//	slist_dump(&format);
	if (format.cnt == 0) {
		fprintf(stderr, "No format loaded - exiting\n");
		return 1;
	}

	if (path)
		au = auparse_init(AUSOURCE_FILE, path);
	else
		au = auparse_init(AUSOURCE_FILE_POINTER, stdin);

	while (auparse_next_event(au) > 0) {
		// For each event, iterate through the format string to
		// see what needs to be output
		snode *s;
		slist_first(&format);
		s = slist_get_cur(&format);
		while (s) {
			print_item(au, s);
			s = slist_next(&format);
		}
	}
	slist_clear(&format);
	return 0;
}

